use crate::helpers::db;
use crate::models::{artists::Artists, spotify};
use crate::schema::tracks;
use chrono::NaiveDateTime;
use diesel::result::Error;
use diesel::{
    AsChangeset, ExpressionMethods, Insertable, QueryDsl, Queryable, RunQueryDsl, Selectable,
};
use serde::{Deserialize, Serialize};

use crate::schema::playlists_tracks;

#[derive(AsChangeset, Clone, Insertable, Queryable, Selectable, Deserialize, Serialize)]
#[diesel(table_name = crate::schema::tracks)]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct Tracks {
    pub id: String,
    pub title: String,
    pub duration_ms: i32,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

impl Tracks {
    pub fn find(id: String) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = tracks::table.filter(tracks::id.eq(id)).first(conn)?;
        Ok(playlist)
    }

    pub fn create(track: Tracks) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(tracks::table)
            .values(Tracks::from(track))
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn find_by_playlist(playlist_id: &str) -> Result<Vec<Tracks>, Error> {
        let conn = &mut db::connection()?;
        let tracks: Vec<(String, String)> = playlists_tracks::table
            .filter(playlists_tracks::playlist_id.eq(playlist_id))
            .get_results::<(String, String)>(conn)?;

        let tracks = tracks
            .into_iter()
            .map(|(_, track_id)| Tracks::find(track_id).unwrap())
            .collect::<Vec<Tracks>>();

        Ok(tracks)
    }

    pub fn create_or_update(track: Tracks) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(tracks::table)
            .values(track.clone())
            .on_conflict(tracks::id)
            .do_update()
            .set(track)
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn get_artists(&self) -> Result<Vec<Artists>, Error> {
        let artists = Artists::find_by_track(&self.id)?;
        Ok(artists)
    }

    fn from(track: Tracks) -> Self {
        Tracks {
            id: track.id,
            title: track.title,
            duration_ms: track.duration_ms,

            created_at: track.created_at,
            updated_at: track.updated_at,

            spotify_id: track.spotify_id,
            tidal_id: track.tidal_id,
        }
    }

    pub fn with_artists(track: Tracks) -> Result<TracksWithArtists, Error> {
        let artists = track.get_artists()?;

        Ok(TracksWithArtists {
            id: track.id,
            title: track.title,
            duration_ms: track.duration_ms,

            artists,

            spotify_id: track.spotify_id,
            tidal_id: track.tidal_id,
        })
    }
}

impl From<spotify::Track> for Tracks {
    fn from(value: spotify::Track) -> Tracks {
        Tracks {
            id: value.external_ids.isrc,
            title: value.name,
            duration_ms: value.duration_ms,
            spotify_id: Some(value.id),
            tidal_id: None,
            created_at: None,
            updated_at: None,
        }
    }
}

#[derive(Deserialize, Serialize)]
pub struct TracksWithArtists {
    pub id: String,
    pub title: String,
    pub duration_ms: i32,

    pub artists: Vec<Artists>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}
