use crate::helpers::db;
use crate::schema::users;
use bcrypt::BcryptError;
use chrono::NaiveDateTime;
use diesel::result::Error;
use diesel::{
    AsChangeset, ExpressionMethods, Insertable, QueryDsl, Queryable, RunQueryDsl, Selectable,
};
use serde::{Deserialize, Serialize};

#[derive(AsChangeset, Insertable, Queryable, Selectable, Deserialize, Serialize)]
#[diesel(table_name = crate::schema::users)]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct User {
    pub id: String,
    pub name: String,
    pub email: String,

    pub password: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,
}

#[derive(Debug, Deserialize, Queryable, Serialize)]
pub struct Users {
    pub id: String,
    pub name: String,
    pub email: String,

    pub password: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,
}

impl Users {
    pub fn find(id: &str) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let user = users::table.filter(users::id.eq(id)).first(conn)?;
        Ok(user)
    }

    pub fn create(user: User) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let user = diesel::insert_into(users::table)
            .values(User::from(user))
            .get_result(conn)?;
        Ok(user)
    }

    pub fn find_by_email(email: &str) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let user = users::table.filter(users::email.eq(email)).first(conn)?;
        Ok(user)
    }

    pub fn verify_password(password: &str, hash: &str) -> Result<bool, BcryptError> {
        bcrypt::verify(password, hash)
    }
}

impl User {
    fn from(user: User) -> User {
        User {
            id: user.id,
            name: user.name,
            email: user.email,
            password: user.password,
            created_at: user.created_at,
            updated_at: user.updated_at,
        }
    }
}
