use crate::{models::spotify::APISearchResponse, services::service::Service, utils::get_env};
use actix_web::http::Method;
use async_trait::async_trait;
use chrono::{Duration, Utc};
use lazy_static::lazy_static;
use reqwest::{Client, RequestBuilder};
use serde::Deserialize;
use std::sync::Mutex;

#[derive(Deserialize)]
struct SpotifyResponse {
    access_token: String,
    expires_in: i64,
}

#[derive(Clone)]
pub struct Spotify {
    token: String,
    expires_at: i64,
}

lazy_static! {
    static ref SPOTIFY_INSTANCE: Mutex<Spotify> = Mutex::new(Spotify::new());
}

pub async fn instance() -> Spotify {
    let mut spotify = SPOTIFY_INSTANCE.lock().unwrap().clone();

    // Fetch token if expired
    spotify.fetch_token().await;

    spotify
}

impl Spotify {
    pub fn new() -> Spotify {
        Spotify {
            token: String::new(),
            expires_at: 0,
        }
    }

    fn api(&self, url: &str, method: Method) -> RequestBuilder {
        let client = Client::new();

        client
            .request(method, format!("https://api.spotify.com/v1{url}"))
            .bearer_auth(self.get_token())
    }

    // pub async fn get_track(self, spotify_id: &str) {
    //     let spotify = &SPOTIFY;
    // }

    // pub async fn get_artist(self, spotify_id: &str) {}

    // pub async fn get_track_by_isrc(self, isrc: &str) {}

    pub async fn search(&self, search: &str) -> Result<APISearchResponse, ()> {
        let response = self
            .api("/search", Method::GET)
            .query(&[("q", search), ("type", "album,artist,track")])
            .send()
            .await;

        match response {
            Ok(res) => {
                let data = res.json::<APISearchResponse>().await;

                match data {
                    Ok(data) => Ok(data),
                    Err(_) => {
                        panic!("Invalid response (Spotify Search)")
                    }
                }
            }
            Err(_) => {
                panic!("Invalid response (Spotify Search)")
            }
        }
    }
}

#[async_trait]
impl Service for Spotify {
    fn get_token(&self) -> &String {
        &self.token
    }

    fn is_expired(&self) -> bool {
        self.expires_at < chrono::Utc::now().timestamp()
    }

    async fn fetch_token(&mut self) {
        if !self.is_expired() {
            return;
        }

        let response = Client::new()
            .post("https://accounts.spotify.com/api/token")
            .basic_auth(
                get_env("SPOTIFY_CLIENT_ID"),
                Some(get_env("SPOTIFY_CLIENT_SECRET")),
            )
            .form(&[("grant_type", "client_credentials")])
            .send()
            .await;

        match response {
            Ok(res) => {
                let data = res.json::<SpotifyResponse>().await;

                match data {
                    Ok(data) => {
                        self.token = data.access_token;
                        self.expires_at =
                            (Utc::now() + Duration::seconds(data.expires_in)).timestamp();
                    }
                    Err(_) => {
                        panic!("Invalid response (Spotify Token)")
                    }
                }
            }
            Err(_) => {
                panic!("Invalid response (Spotify Token)")
            }
        }
    }
}
